"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*******************************************************************************
 * Originally from GulpRevNapkin.
 * Delete Original files left from gulp-rev(-all).
 * Patched as npm package hasn't been updated... :(
 ******************************************************************************/
var chalk_1 = require("chalk");
var log = require("fancy-log");
var fs = require("fs");
var path = require("path");
var PluginError = require("plugin-error");
var rimraf = require("rimraf");
var through = require("through2");
var internal_1 = require("../internal");
// CONSTANT
var PLUGIN_NAME = 'gulp-rev-clean';
var createError = function (msg) {
    return new PluginError({
        message: msg,
        plugin: PLUGIN_NAME,
    });
};
/**
 * gulp-rev-napkin
 * @param {GulpRevNapkinOpts} [options] - configuration
 * @return - NodeJS Transform Stream
 */
exports.revNapkin = function (options) {
    // Validate Input
    if (options) {
        if (!(options instanceof Object)) {
            throw createError('<options> must be typeof "object"');
        }
        if (options.verbose && typeof options.verbose !== 'boolean') {
            throw createError('<options>.verbose must be typeof "boolean"');
        }
        if (options.force && typeof options.force !== 'boolean') {
            throw createError('<options>.force must be typeof "boolean"');
        }
    }
    // Defaults
    options = options || {};
    if (options.verbose === undefined || options.verbose === null) {
        options.verbose = true;
    }
    options.verbose = options.verbose ? true : false;
    options.force = options.force ? true : false;
    // Napkin
    var stream = through.obj(function (file, enc, callback) {
        this.push(file);
        if (file.isNull() || !file.revOrigPath) {
            return callback();
        }
        function rimrafHandler(error) {
            if (error) {
                callback(createError(error.message));
                return;
            }
            log(chalk_1.default.red('DELETED:'), '"' + chalk_1.default.cyan(file.revOrigPath) + '"');
            callback();
        }
        function lstatHandler(error, stats) {
            if (error) {
                callback(createError(error.message));
                return;
            }
            if (stats.isDirectory()) {
                callback(createError('Directory removal NOT supported "' + file.revOrigPath + '"'));
                return;
            }
            // For Safety: Resolve Paths
            var cwd = file.cwd || internal_1.TIMPLA_PROCESS.PROCESS_CWD;
            var filepath = path.resolve(cwd, file.revOrigPath);
            var relativeFromCwd = path.relative(cwd, filepath);
            // Restrict to CWD: unless specified
            if (!options.force && relativeFromCwd.substr(0, 2) === '..') {
                callback(createError('Delete outside of CWD NOT supported. To enable -> {force:true}: "' +
                    file.revOrigPath +
                    '"'));
            }
            else {
                rimraf(file.revOrigPath, rimrafHandler);
            }
        }
        fs.lstat(file.revOrigPath, lstatHandler);
    });
    return stream;
};
//# sourceMappingURL=revNapkin.js.map