"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var code_frame_1 = require("@babel/code-frame");
var chalk_1 = require("chalk");
var debug = require("debug");
var deepmerge = require("deepmerge");
var log = require("fancy-log");
var fs = require("fs");
var notify = require("gulp-notify");
var ImportLazy = require("import-lazy");
var logSymbols = require("log-symbols");
var os = require("os");
var path = require("path");
/** An all-purpose separator */
var separator = "==================================================";
/** DEBUG=timpla logger */
exports.devLogger = debug('timpla');
/** Logs the given list of strings to the console. */
exports.devLoggerTitle = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    exports.devLogger(os.EOL);
    exports.devLogger(chalk_1.default.green(separator));
    args.forEach(function (t) { return exports.devLogger(chalk_1.default.green(t)); });
    exports.devLogger(chalk_1.default.green(separator));
    exports.devLogger(os.EOL);
};
/** Logs the given list of strings to the console. */
exports.logTitle = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    log(os.EOL);
    log(chalk_1.default.green(separator));
    args.forEach(function (t) { return log(chalk_1.default.green(t)); });
    log(chalk_1.default.green(separator));
    log(os.EOL);
};
exports.pathToUrl = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return path.join.apply(path, __spread(args)).replace(/\\/g, '/');
};
var colors = new chalk_1.default.constructor({ enabled: true });
var errorDisplayHelper = {
    error: {
        messageColor: colors.red,
        severityText: 'error',
        symbol: logSymbols.error,
    },
    warning: {
        messageColor: colors.yellow,
        severityText: 'warning',
        symbol: logSymbols.warning,
    },
};
var getFrameColumns = function (rawLines, message) {
    return code_frame_1.codeFrameColumns(rawLines, { start: { line: message.line, column: message.character } }, { highlightCode: true })
        .split('\n')
        .map(function (str) { return "  " + str; })
        .join(os.EOL);
};
var getWbMessage = function (_a) {
    var frame = _a.frame, fullMessage = _a.fullMessage, symbol = _a.symbol, messageColor = _a.messageColor, severityText = _a.severityText, ruleName = _a.ruleName;
    return [
        [
            messageColor.bold("Type " + severityText + ": "),
            fullMessage,
            '  ',
            symbol,
            messageColor.underline(ruleName),
        ].join(' '),
        frame && '',
        frame,
        frame && '',
    ]
        .filter(function (e) { return !!e || e === ''; })
        .join(os.EOL);
};
exports.esWbformatter = function (_a) {
    var basePath = _a.basePath;
    return function (results) {
        if (!results || !results.length) {
            return '';
        }
        var errorCount = 0;
        var warningCount = 0;
        var filesOutput = results.map(function (result) {
            if (!result.messages || !result.messages.length) {
                return;
            }
            var rawLines = fs.readFileSync(result.filePath, 'utf8');
            var numLines = rawLines.split('\n').length - 1;
            var messagesOutput = result.messages.map(function (message) {
                var isError = message.severity === 2;
                var selector = isError ? 'error' : 'warning';
                var _a = errorDisplayHelper[selector], messageColor = _a.messageColor, severityText = _a.severityText, symbol = _a.symbol;
                isError ? warningCount++ : errorCount++;
                // ESLint goes a bit crazy at times
                // If the current line returned in the message is more, we simply don't generate a code frame
                var generateFrame = message.line <= numLines;
                return getWbMessage({
                    frame: generateFrame && getFrameColumns(rawLines, message),
                    fullMessage: message.message,
                    messageColor: messageColor,
                    ruleName: "ES " + message.ruleId,
                    severityText: severityText,
                    symbol: symbol,
                });
            });
            var filename = chalk_1.default.underline(path.relative(basePath, result.filePath));
            return [filename, messagesOutput.join(os.EOL)].join(os.EOL);
        });
        var finalOutput = [
            '',
            filesOutput.filter(function (s) { return s; }).join(os.EOL),
            errorCount && chalk_1.default.red("Error/s: " + errorCount),
            warningCount && chalk_1.default.yellow("Warning/s: " + warningCount),
            '',
        ]
            .filter(Boolean)
            .join(os.EOL);
        return errorCount + warningCount > 0 ? finalOutput : '';
    };
};
/** Formatter for ForkTSCheckerWebpack */
exports.tsWbformatter = function (message) {
    var rawLines = fs.readFileSync(message.file, 'utf-8');
    var isError = !message.isWarningSeverity();
    var selector = isError ? 'error' : 'warning';
    var _a = errorDisplayHelper[selector], messageColor = _a.messageColor, severityText = _a.severityText, symbol = _a.symbol;
    return getWbMessage({
        frame: getFrameColumns(rawLines, message),
        fullMessage: message.content,
        messageColor: messageColor,
        ruleName: "TS " + message.code,
        severityText: severityText,
        symbol: symbol,
    });
};
/** Provides empty defaults for empty objects or arrays */
var getEmptyTarget = function (value) { return (Array.isArray(value) ? [] : {}); };
/** Normalises objects for merge. */
var clone = function (value, options) { return deepmerge(getEmptyTarget(value), value, options); };
/**
 * Recursively combines two objects.
 * Nested arrays are merged as well.
 */
exports.arrayMerge = function (target, source, options) {
    var destination = target.slice();
    source.forEach(function (e, i) {
        if (typeof destination[i] === 'undefined') {
            var cloneRequested = options.clone !== false;
            var shouldClone = cloneRequested && options.isMergeableObject(e);
            destination[i] = shouldClone ? clone(e, options) : e;
        }
        else if (options.isMergeableObject(e)) {
            destination[i] = deepmerge(target[i], e, options);
        }
        else if (target.indexOf(e) === -1) {
            destination.push(e);
        }
    });
    return destination;
};
/**
 * Finds the first valid object from a list of arguments.
 */
exports.getFirstValidObject = function () {
    var objs = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        objs[_i] = arguments[_i];
    }
    return objs.find(function (obj) { return typeof obj === 'object'; });
};
/**
 * Handles gulp errors.
 */
exports.handleErrors = function (errorObject) {
    notify
        .onError(errorObject
        .toString()
        .split(': ')
        .join(':\n'))
        .apply(this, arguments);
    // Keep gulp from hanging on this task
    if (this.emit === 'function') {
        this.emit('end');
    }
};
/**
 * Adds displayNames to functions.
 * Useful for debugging.
 */
exports.setDisplayName = function (fun, name) {
    if (['function', 'object'].includes(typeof fun)) {
        fun.displayName = name;
    }
    return fun;
};
/** Lazily imports packages */
exports.sureLazyImport = ImportLazy(require);
/** Checks if a folder is a child of the specified parent */
exports.isChildOf = function (child, parent) {
    if (child === parent) {
        return false;
    }
    var parentTokens = parent.split(path.sep).filter(function (i) { return i.length; });
    return parentTokens.every(function (t, i) { return child.split('/')[i] === t; });
};
/**
 * Checks if gulp tasks exist.
 * If not, throws an error to notify the user to specify the right config.
 * Also sets the displayName for logging
 */
exports.checkAndLabel = function (fun, name) {
    if (typeof fun !== 'function') {
        throw Error(chalk_1.default.red("Please ensure that the alternative task for " + chalk_1.default.yellow(name) + " returns a function that either returns a gulp stream, or manually calls a callback() to signal completion."));
    }
    exports.setDisplayName(fun, name);
};
//# sourceMappingURL=utils.js.map